#!/usr/bin/env python

from argparse import ArgumentParser
import os
import sys
import urllib.request
try:
    from . import fingerling as fl
except:
    import fingerling as fl

VERSION = '0.2.0'

def parse_arguments():
    desc = 'Xiami music downloader.'
    parser = ArgumentParser(description=desc)
    parser.add_argument('type', choices=['single', 'album', 'omnibus'],
                        help="type")
    parser.add_argument('id', type=int,  help='id in the url')
    parser.add_argument('-nt', '--no-tag', action='store_true',
                        help='skip adding ID3 tag')
    parser.add_argument('-d', '--directory', default='',
                        help='specify the directory to store files;' + \
                        ' current directory by default')
    parser.add_argument('-v', '--version', action='version', version=VERSION)
    return parser.parse_args()


def download(url, filename):
    parent_dir = os.path.abspath(os.path.dirname(filename))
    if not os.path.exists(parent_dir):
        os.makedirs(parent_dir)
    urllib.request.urlretrieve(url, filename)


def main():
    args = parse_arguments()

    urls = []
    urls.append(fl.gen_tracklist_urls(args.type, args.id))

    tracklist = []
    for url in urls:
        tracklist += fl.get_tracklist_from_url(url)

    for i in range(0, len(tracklist)):
        tracklist[i]['location'] = fl.decrypt_location(tracklist[i]['location'])

    sys.stdout.write('%d file(s) to download\n' % len(tracklist))

    for i, track in enumerate(tracklist):
        base_dir = args.directory
        dest_file_path = os.path.join(base_dir,
                           '{title}-{artist}.mp3'.format(**track))

        progress = '[{num}/{total}] {filepath}\n'.format(num=i+1,
                                                         total=len(tracklist),
                                                         filepath=dest_file_path)
        sys.stdout.write(progress)
        download(track['location'], dest_file_path)

        if not args.no_tag:
            fl.add_id3_tag(dest_file_path, track)

if __name__ == '__main__':
    main()
